
#include "mailserv.h"

#define HEAD1		27		/* Field width of first third	*/
#define HEAD2		20		/* Field width of second third	*/
#define HEAD3		21		/* Field width of last third	*/
#define DATELEN		16		/* Amount of date to put in hdr	*/
#define LINELEN		70		/* Length of an average line	*/
#define MAXMSGS		64		/* Maximum number of msgs/digest*/
#define LINESIZE	256		/* Maximum line size		*/

struct message 
{
    char *Date;
    char *From;
    char *Subject;
    char *MessageID;
    char *ReprintFrom;
    char *sortstring;
    long messageaddr;
    long messagelength;
    int  use_re;
} messages[MAXMSGS];

FILE *input;
FILE *output;

int digestsize = 0;
int nmessages = 0;

char *safter(), *nospace(), *getline();
char digestsubject[100];

/*****************************************************************************
 * vol_issue_out
 */
void
    vol_issue_out ()
{
    FILE *voliss;

    voliss = Fopen (listinfo.digestcounterfile, "w");
    fprintf (voliss, "%d %d\n", listinfo.volume, (listinfo.issue)+1);
    fclose (voliss);
}

/*****************************************************************************
 * vol_issue_info
 */
void
    vol_issue_info()
{
    char templine[100];
    FILE *voliss;
    
    voliss = Fopen (listinfo.digestcounterfile, "r");
    fscanf (voliss, "%d %d\n", &listinfo.volume, &listinfo.issue);
    fclose (voliss);

    if ((listinfo.datestr[8] == ' ') &&
	(listinfo.datestr[9] == '1'))
    {
	listinfo.volume += 1;
	listinfo.issue = 1;
    }

    sprintf (templine, "Volume %d: Issue %3d",
	     listinfo.volume, listinfo.issue);
    listinfo.volline = strdup (templine);
}

/*****************************************************************************
 * distribute_digest
 */
void
    distribute_digest()
{
    char cmdstr[120];
    int i;

    for (i=0; i<readercount; i++)
    {
	sprintf (cmdstr, "%s %s < %s",
		 SendMail, readers[i], listinfo.digestoutput);
	system (cmdstr);
	sleep (listinfo.delay);
    }
}

/*****************************************************************************
 * make_digest
 */
void
    make_digest()
{
    char cmdline[200];

    sprintf (cmdline, "%s %s %s.prev", Copy,
	     listinfo.digestinput, listinfo.digestinput);
    system (cmdline);

    vol_issue_info();
    vol_issue_out();
    
    fprintf(thelogfile, 
	    "Assembling %s Digest %s (%.*s)\n", 
	    listinfo.digestname, listinfo.volline, DATELEN, 
	    listinfo.datestr);
    fprintf(thelogfile, "Scanning and sorting messages for topic lines.\n");

    scan_messages();

    if (nmessages == 0)
    {
	listinfo.issue -= 1;
	vol_issue_out();
	fprintf (thelogfile, "Zero length digest, not mailing.\n");
	listinfo.ftp = False;
	listinfo.keep = False;
	return;
    }

    sort_messages();
    
    fprintf (thelogfile, "Writing %s Digest to \"%s\"\n",
	     listinfo.digestname, listinfo.digestoutput);
    
    do_digest_header();
    read_messages();
    
    fprintf (thelogfile, "The digest is %d characters long in %d messages.\n",
	     digestsize, nmessages);

    distribute_digest();
}

/*****************************************************************************
 * getline
 */
char *getline(fp)
FILE *fp;
{
    register int c;
    register char *str, *str_begin;
    
    if ((str = malloc(LINESIZE)) == NULL) 
    {
	fprintf (thelogfile, "digest: out of memory.\n");
	Exit(1);
    }
    
    str_begin = str;
    
    while (((str - str_begin) < (LINESIZE - 1)) &&
	   ((c = getc(fp)) != '\n') && (c != EOF))
	*str++ = c;

    *str++ = NULL;
    
    if (c == EOF)
	return(NULL);
    
    return(str_begin);
}

/*****************************************************************************
 * scan_messages
 */
scan_messages()
{
    register long n;
    register char *s;
    
    input = Fopen(listinfo.digestinput, "r");

    /*
     * We break out of this from inside.
     */
    for (;;) 
    {
	if (nmessages >= MAXMSGS) 
	{
	    fprintf (thelogfile, "digest: too many messages.\n");
	    Exit(1);
	}
	
	/*
	 * Find the start of the next message.
	 */
	do {
	    /*
	     * If we hit EOF, mark the length of the
	     * previous message and go back.
	     */
	    if ((s = getline(input)) == NULL) {
		n = ftell(input);
		n = n - messages[nmessages - 1].messageaddr;
		messages[nmessages - 1].messagelength = n;
		return;
	    }
	} while (strncmp(s, "From ", 5) != 0);
	
	/*
	 * If we have found another message, mark the
	 * length of the previous message.
	 */
	if (nmessages) {
	    n = ftell(input);
	    n = n - (strlen(s) + 1);
	    n = n - messages[nmessages - 1].messageaddr;
	    messages[nmessages - 1].messagelength = n;
	}
	
	/*
	 * Read in the headers.
	 */
	for (;;) {
	    /*
	     * We shouldn't hit EOF here, we should
	     * at least finish the headers first.
	     */
	    if ((s = getline(input)) == NULL) {
		fprintf (thelogfile, "digest: \"%s\": unexpected EOF.\n",
			 listinfo.digestinput);
		Exit(1);
	    }
	    
	    /*
	     * Blank line terminates headers.
	     */
	    if (*s == NULL)
		break;
	    
	    /*
	     * Save certain headers.  We strip the
	     * header name and leading whitespace.
	     */
	    if (strncmp(s, "Message-ID:", 11) == 0) {
		messages[nmessages].MessageID = 
		    nospace(safter(s, "Message-ID:"));
	    }
	    else if (strncmp(s, "Message-Id:", 11) == 0) {
		messages[nmessages].MessageID = 
		    nospace(safter(s, "Message-Id:"));
	    }
	    else if (strncmp(s, "From:", 5) == 0) {
		messages[nmessages].From = 
		    nospace(safter(s, "From:"));
	    }
	    else if (strncmp(s, "Newsgroups:", 11) == 0) {
		messages[nmessages].ReprintFrom = 
		    nospace(safter(s, "Newsgroups:"));
	    }
	    else if (strncmp(s, "Date:", 5) == 0) {
		messages[nmessages].Date = 
		    nospace(safter(s, "Date:"));
	    }
	    else if (strncmp(s, "Subject:", 8) == 0) {
		s = nospace(safter(s, "Subject:"));
		
		/*
		 * We don't need the "Re:" stuff.
		 */
		messages[nmessages].use_re = 0;
		while ((strncmp(s, "re:", 3) == 0) || 
		       (strncmp(s, "Re:", 3) == 0) ||
		       (strncmp(s, "RE:", 3) == 0) || 
		       (strncmp(s, "rE:", 3) == 0)) 
		{
		    messages[nmessages].use_re = 1;
		    s += 3;
		    s = nospace(s);
		}
		
		messages[nmessages].Subject = s;
	    }
	    else {
		/*
		 * If we aren't saving this line,
		 * give the memory back.
		 */
		free(s);
	    }
	}
	
	/*
	 * The message starts after the header.
	 */
	messages[nmessages].messageaddr = ftell(input);
	nmessages++;
    }
}

/*****************************************************************************
 * sort_messages
 */
sort_messages()
{
    register int i;
    extern int comp();
    register char *s, *t;
    
    for (i=0; i < nmessages; i++) {
	/*
	 * Skip messages with no subject.
	 */
	if (messages[i].Subject == NULL)
	    continue;
	
	s = messages[i].Subject;
	
	if ((t = malloc(strlen(s)+1)) == NULL) {
	    fprintf (thelogfile, "digest: out of memory.\n");
	    Exit(1);
	}
	
	messages[i].sortstring = t;
	
	/*
	 * Zap leading whitespace.
	 */
	s = nospace(s);
	
	/*
	 * Copy the subject string into sortstring
	 * converting upper case to lower case and
	 * ignoring whitespace.
	 */
	while (*s) {
	    if ((*s == ' ') || (*s == '\t')) {
		s++;
		continue;
	    }
	    
	    if (isupper(*s))
		*t++ = tolower(*s);
	    else
		*t++ = *s;
	    
	    s++;
	}
	
	*t = NULL;
    }
    
    /*
     * Sort 'em.
     */
    qsort(messages, nmessages, sizeof(struct message), comp);
}

/*****************************************************************************
 * comp
 */
comp(m1, m2)
register struct message *m1, *m2;
{
    int admin1, admin2;
    
    if (m1->sortstring == NULL) {
	if (m2->sortstring == NULL)
	    return(0);
	return(1);		/* no subject messages to end */
    }
    
    if (m2->sortstring == NULL)
	return(-1);		/* no subject messages to end */
    
    admin1 = strncmp(m1->sortstring, "administrivia", 13);
    admin2 = strncmp(m2->sortstring, "administrivia", 13);
    
    if (admin1 == 0) {
	if (admin2 == 0)
	    return(0);
	return(-1);		/* administrivia to beginning */
    }
    
    if (admin2 == 0)
	return(1);		/* administrivia to beginning */
    
    return(strcmp(m1->sortstring, m2->sortstring));
}

/*****************************************************************************
 * do_digest_header
 */
do_digest_header()
{
    FILE *fp;
    char *laststr;
    char buf[BUFSIZ];
    char tmp[LINESIZE];
    extern int comp2();
    register int i, j, length;
    char *c;
    
    output = Fopen(listinfo.digestoutput, "w");
    digestsize = 0;
    
    /*
     * Mailer headers.
     */
    sprintf (buf, "Precedence: Bulk\n", listinfo.datestr);
    digestsize += strlen (buf);
    fputs(buf, output);

    sprintf(buf, "Date: %s\n", listinfo.datestr);
    digestsize += strlen(buf);
    fputs(buf, output);
    
    sprintf(buf, "Errors-To: %s\n", listinfo.errorsto);
    digestsize += strlen(buf);
    fputs(buf, output);
    
    sprintf(buf, "From: %s\n", listinfo.from);
    digestsize += strlen(buf);
    fputs(buf, output);
    
    sprintf(buf, "Reply-To: %s\n", listinfo.replyto);
    digestsize += strlen(buf);
    fputs(buf, output);
    
    strcpy (tmp, listinfo.volline);
    for (c = tmp; *c != 0 && *c != ':'; c++) ;

    do 
    { *c-- = 0; } 
    while (c >= tmp && *c == ' ');
    
    do 
    { c--; } 
    while (c >= tmp && *c != ' ');

    c++;
    digestsize += 9; fputs ("Subject: ", output);
    sprintf(digestsubject, "%s Digest V%s #%d",
	    listinfo.digestname, c, listinfo.issue);
    digestsize += strlen(digestsubject);
    fputs(digestsubject, output);
    digestsize++, putc ('\n', output);
    
    /*
     * The digest header.
     */
    sprintf(tmp, "%s Digest", listinfo.digestname);
    sprintf(buf, "\n\n%-*.*s %-*.*s %-*.*s\n\n",
	    HEAD1, HEAD1, tmp,
	    HEAD2, DATELEN, listinfo.datestr,
	    HEAD3, HEAD3, listinfo.volline);
    digestsize += strlen(buf);
    fputs(buf, output);
    
    sprintf(buf, "Today's Topics:\n");
    digestsize += strlen(buf);
    fputs(buf, output);
    
    /*
     * Do today's topics lines.
     */
    laststr = "";
    for (i=0; i < nmessages; i++) {
	/*
	 * No topic.
	 */
	if (messages[i].Subject == NULL)
	    continue;
	
	laststr = messages[i].sortstring;
	
	/*
	 * Count the number of messages with this topic.
	 */
	j = 1;
	while (((i + j) < nmessages)
	       && (messages[i+j].sortstring != NULL)
	       && (strcmp(laststr, messages[i+j].sortstring) == 0))
	    j++;
	
	/*
	 * Print the topic centered on the line.
	 */
	if (j > 1) {
	    sprintf(tmp, "%s (%d msgs)", messages[i].Subject, j);
	    length = (LINELEN / 2) + (strlen(tmp) / 2);
	    sprintf(buf, "%*s\n", length, tmp);
	    
	    /*
	     * Sort messages with same topic into their
	     * original arrival order.
	     */
	    qsort(&messages[i], j, sizeof(struct message), comp2);
	    i += (j - 1);
	}
	else {
	    length = (LINELEN / 2) + (strlen(messages[i].Subject) / 2);
	    sprintf(buf, "%*s\n", length, messages[i].Subject);
	}
	
	digestsize += strlen(buf);
	fputs(buf, output);
    }
    
    /*
     * Read the listinfo.digesthead file, if there is one.
     */
    fp = Fopen(listinfo.digesthead, "r");

    putc('\n', output);
    digestsize++;
    
    while (fgets(buf, BUFSIZ, fp) != NULL) {
	digestsize += strlen(buf);
	fputs(buf, output);
    }
    
    fclose(fp);
    putc('\n', output);

    /*
     * Print a line of dashes.
     */
    for (i=0; i < LINELEN; i++) {
	putc('-', output);
	digestsize++;
    }
    
    fputs("\n\n", output);
    digestsize += 2;
}

/*****************************************************************************
 * comp2
 */
comp2(m1, m2)
register struct message *m1, *m2;
{
    return(m1->messageaddr - m2->messageaddr);
}

/*****************************************************************************
 * read_messages
 */
read_messages()
{
    FILE *fp;
    char buf[BUFSIZ];
    register char *s, *t;
    register int i, length;
    int linestart;
    
    for (i=0; i < nmessages; i++) {
	/*
	 * Just in case.
	 */
	clearerr(input);
	
	/*
	 * Put the message's headers back in.
	 */
	sprintf(buf, "Date: %s\n", messages[i].Date);
	digestsize += strlen(buf);
	fputs(buf, output);
	
	sprintf(buf, "From: %s\n", messages[i].From);
	digestsize += strlen(buf);
	fputs(buf, output);
	
	if (messages[i].Subject != NULL) {
	    sprintf(buf, "Subject: %s%s\n",
		    (messages[i].use_re ? "Re: " : ""),
		    messages[i].Subject);
	    digestsize += strlen(buf);
	    fputs(buf, output);
	}
	
	if (messages[i].MessageID != NULL) {
	    sprintf(buf, "Message-ID: %s\n", messages[i].MessageID);
	    digestsize += strlen(buf);
	    fputs(buf, output);
	}
	
	if (messages[i].ReprintFrom != NULL) {
	    sprintf (buf, "\nReprintFrom: %s\n", messages[i].ReprintFrom);
	    digestsize += strlen(buf);
	    fputs (buf, output);
	}
	
	putc ('\n', output); digestsize++;
	
	/*
	 * Read the message into memory.  This is
	 * so we can zap extra blank lines.
	 */
	fseek(input, messages[i].messageaddr, 0);
	length = messages[i].messagelength;
	
	if ((s = malloc(length+1)) == NULL) {
	    fprintf (thelogfile, "digest: out of memory.\n");
	    Exit(1);
	}
	
	fread(s, 1, length, input);
	
	/*
	 * Zap trailing newlines.
	 */
	t = s + length;
	while (length > 0 && *--t == '\n')
	    length--;
	*++t = NULL;
	
	/*
	 * Zap leading newlines.
	 */
	t = s;
	while (*t++ == '\n')
	    length--;
	t--;
	
	/*
	 * Write the message.
	 *
	 * In any line which begins with 30 or more hyphens,
	 * change the first hyphen to a space, to avoid a clash
	 * with the message separator string.
	 */
	digestsize += length;
	linestart = 1;
	while (length-- > 0) {
	    if (linestart &&
		strncmp(t,"------------------------------",30) == 0)
		linestart = 0, t++, putc (' ', output);
	    else {
		linestart = (*t == '\n');
		putc (*t++, output);
	    }
	}
	
	sprintf(buf, "\n\n------------------------------\n\n");
	digestsize += strlen(buf);
	fputs(buf, output);
	free(s);
    }
    
    /*
     * All done.
     */
    sprintf(buf, "End of %s\n******************************",
	    digestsubject);
    digestsize += strlen(buf);
    fputs(buf, output);
    for (i = strlen (digestsubject) + 7; i > 30; i--)
	digestsize++, putc ('*', output);
    digestsize++, putc ('\n', output);

    fp = Fopen(listinfo.digesttail, "r");
    putc('\n', output);
    digestsize++;
    
    while (fgets(buf, BUFSIZ, fp) != NULL) {
	digestsize += strlen(buf);
	fputs(buf, output);
    }
    
    fclose(fp);
    putc('\n', output);

    fflush (output);
    fflush (output);
    fclose(output);

    fclose(input);
    input = Fopen (listinfo.digestinput, "w");
    fclose (input);
}

/*
 * safter - return a pointer to the position in str which follows pat.
 */
char *safter(str, pat)
register char *str, *pat;
{
    register int len;
    
    len = strlen(pat);
    
    while (*str) {
	if (strncmp(str, pat, len) == 0) {
	    str += len;
	    return(str);
	}
	
	str++;
    }
    
    return(NULL);
}

/*
 * nospace - advance s over leading whitespace, return new value.
 */
char *nospace(s)
register char *s;
{
    while ((*s != NULL) && ((*s == ' ') || (*s == '\t')))
	s++;
    
    return(s);
}

