
#include "mailserv.h"

/*****************************************************************************
 * expand_path
 *
 * If a file's path is not explicitly global or relative, make it global
 * by prepending MailServe's home directory and the list directory.
 */
char
    *expand_path (fname)
char *fname;
{
    char *carp;

    if ((fname[0] != '/') && (fname[0] != '.'))
    {
	carp = (char *) malloc (strlen (ListHome) + strlen (listinfo.name) +
				strlen (fname) + 5);
	sprintf (carp, "%s/%s/%s", ListHome, listinfo.name, fname);
	return (carp);
    }

    return (strdup (fname));
}

/*****************************************************************************
 * boolean_info_line
 *
 * Given a line from the info file expecting a boolean value, determine
 * the setting.
 */
int
    boolean_info_line (chptr)
char *chptr;
{
    char cmdword[20], flag[5], *carp;

    sscanf (chptr, "%s = %s", &cmdword, &flag);
    if (carp = strstr (flag, "y"))
	return (True);
    else
	return (False);
}

/*****************************************************************************
 * val_info_line
 *
 * Given a line from the info file expecting an integer value, determine
 * the amount.
 */
int
    val_info_line (chptr)
char *chptr;
{
    char cmdword[20], value[5];

    sscanf (chptr, "%s = %s", &cmdword, &value);
    return (atoi (value));
}

/*****************************************************************************
 * str_info_line
 *
 * Given a line from the info file expecting a string, extract it.
 */
char
    *str_info_line (chptr)
char *chptr;
{
    return (cleanup_string (strstr (chptr, "=") + 1));
}

/*****************************************************************************
 * file_info_line
 *
 * Given a line from the info file expecting a string that is a file name,
 * extract it.
 */
char
    *file_info_line (chptr)
char *chptr;
{
    char *carp1, *carp2;

    carp1 = str_info_line (chptr);
    carp2 = expand_path (carp1);
    free (carp1);
    return (carp2);
}

/*****************************************************************************
 * get_list_info
 *
 * Parse the info file.  This function should probably be rewritten in some
 * more useful parsing method.
 */
void
    get_list_info (argc, argv)
int argc;
char **argv;
{
    char filepath[100], templine[100], *carp;
    int lnum = 0;
    FILE *infofile;

    if (argc != 3)
    {
	usage ();
    }

    listinfo.name = strdup (argv[1]);
    listinfo.sub = strdup (argv[2]);

    sprintf (filepath, "%s/%s/%s", ListHome, listinfo.name, InfoFile);

    infofile = Fopen (filepath, "r");

    while (fgets (templine, 100, infofile) != NULL)
    {
	lnum ++;
	if ((templine[0] != '#') && (strlen (templine)) > 3)
	{
	    /*
	     * Booleans
	     */
	    if ((carp = strstr (templine, "keep ")) && (carp == templine))
		listinfo.keep = boolean_info_line (carp);

	    else if ((carp = strstr (templine, "announce ")) && (carp == templine))
		listinfo.announce = boolean_info_line (carp);

	    else if ((carp = strstr (templine, "files ")) && (carp == templine))
		listinfo.files = boolean_info_line (carp);

	    else if ((carp = strstr (templine, "digest ")) && (carp == templine))
		listinfo.digest = boolean_info_line (carp);

	    else if ((carp = strstr (templine, "backup ")) && (carp == templine))
		listinfo.backup = boolean_info_line (carp);

	    else if ((carp = strstr (templine, "faq ")) && (carp == templine))
		listinfo.faq = boolean_info_line (carp);

	    else if ((carp = strstr (templine, "ftp ")) && (carp == templine))
		listinfo.ftp = boolean_info_line (carp);

	    /* 
	     * Values
	     */
	    else if ((carp = strstr (templine, "delay ")) && (carp == templine))
		listinfo.delay = val_info_line (carp);

	    /* 
	     * Strings - general
	     */
	    else if ((carp = strstr (templine, "replyto ")) && (carp == templine))
		listinfo.replyto = (char *) str_info_line (carp);

	    else if ((carp = strstr (templine, "errorsto ")) && (carp == templine))
		listinfo.errorsto = (char *) str_info_line (carp);

	    else if ((carp = strstr (templine, "from ")) && (carp == templine))
		listinfo.from = (char *) str_info_line (carp);

	    else if ((carp = strstr (templine, "ftpsite ")) && (carp == templine))
		listinfo.ftpsite = (char *) str_info_line (carp);

	    else if ((carp = strstr (templine, "ftpdir ")) && (carp == templine))
		listinfo.ftpdir = (char *) str_info_line (carp);

	    else if ((carp = strstr (templine, "digestname ")) && (carp == templine))
		listinfo.digestname = (char *) str_info_line (carp);

	    /* 
	     * Strings - filenames/paths
	     */
	    else if ((carp = strstr (templine, "archivefile ")) && (carp == templine))
		listinfo.archivefile = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "userfile ")) && (carp == templine))
		listinfo.userfile = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "logfile ")) && (carp == templine))
		listinfo.logfile = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "indexfile ")) && (carp == templine))
		listinfo.indexfile = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "backupdir")) && (carp == templine))
		listinfo.backupdir = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "subscribefile ")) && (carp == templine))
		listinfo.subscribefile = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "unsubscribefile ")) && (carp == templine))
		listinfo.unsubscribefile = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "nounsubfile ")) && (carp == templine))
		listinfo.nounsubfile = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "helpfile ")) && (carp == templine))
		listinfo.helpfile = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "faqfile ")) && (carp == templine))
		listinfo.faqfile = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "netrcfile ")) && (carp == templine))
		listinfo.netrcfile = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "digestinput ")) && (carp == templine))
		listinfo.digestinput = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "digestoutput ")) && (carp == templine))
		listinfo.digestoutput = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "digestcounterfile ")) && (carp == templine))
		listinfo.digestcounterfile = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "digesthead ")) && (carp == templine))
		listinfo.digesthead = (char *) file_info_line (carp);

	    else if ((carp = strstr (templine, "digesttail ")) && (carp == templine))
		listinfo.digesttail = (char *) file_info_line (carp);

	    else
	    {
		fprintf (stderr, "Error on line %d of %s\n",
			 lnum, filepath);
		fprintf (thelogfile, "Error on line %d of %s @ %s\n",
			 lnum, filepath, listinfo.datestr);
		Exit (1);
	    }
	}
    }

    fclose (infofile);
}

int okinfo;

/*****************************************************************************
 * badinfo
 *
 * Workhorse for logging errors in the info file.
 */
void
    badinfo (carp)
char *carp;
{
    fprintf (stderr, "Error in Info file: %s not defined\n", carp);
    fprintf (thelogfile, "Error in Info file: %s not defined\n", carp);
    okinfo = False;
}

/*****************************************************************************
 * verify_info
 *
 * Check to see that the entried in the info file are legal.
 */
void
    verify_info ()
{
    okinfo = True;

    if (listinfo.logfile == NULL)
    {
	fprintf (stderr, "Error in Info file: logfile not defined\n");
	fflush (stderr);
	exit (1);
    }
    thelogfile = Fopen (listinfo.logfile, "a");

    if (listinfo.replyto == NULL)
	badinfo ("replyto");
    if (listinfo.errorsto == NULL)
	badinfo ("errorsto");
    if (listinfo.userfile == NULL)
	badinfo ("userfile");
    if (listinfo.delay == NULL)
	badinfo ("delay");

    if (listinfo.from != NULL)
    {
	if (listinfo.subscribefile == NULL)
	    badinfo ("subscribefile");
	if (listinfo.unsubscribefile == NULL)
	    badinfo ("unsubscribefile");
	if (listinfo.nounsubfile == NULL)
	    badinfo ("nounsubfile");
	if (listinfo.helpfile == NULL)
	    badinfo ("helpfile");
    }

    if (listinfo.files)
    {
	if (listinfo.indexfile == NULL)
	    badinfo ("indexfile");
    }

    if (listinfo.keep)
    {
	if (listinfo.archivefile == NULL)
	    badinfo ("archivefile");
    }

    if (listinfo.digest)
    {
	if (listinfo.digestinput == NULL)
	    badinfo ("digestinput");
	if (listinfo.digestoutput == NULL)
	    badinfo ("digestoutput");
	if (listinfo.digestcounterfile == NULL)
	    badinfo ("digestcounterfile");
	if (listinfo.digestname == NULL)
	    badinfo ("digestname");
	if (listinfo.digesthead == NULL)
	    badinfo ("digesthead");
	if (listinfo.digesttail == NULL)
	    badinfo ("digesttail");

	if (listinfo.ftp)
	{
	    if (listinfo.ftpdir == NULL)
		badinfo ("ftpdir");
	    if (listinfo.ftpsite == NULL)
		badinfo ("ftpsite");
	    if (listinfo.netrcfile == NULL)
		badinfo ("netrcfile");
	}
    }

    if (listinfo.backup)
    {
	if (listinfo.backupdir == NULL)
	    badinfo ("backupdir");
    }

    if (listinfo.faq)
    {
	if (listinfo.faqfile == NULL)
	    badinfo ("backupdir");
    }

    if (!okinfo)
	Exit (1);
}

