
#include "mailserv.h"

/*****************************************************************************
 * get_raw_msg
 *
 * Load the incoming message into a buffer for processing.
 */
void
    get_raw_msg()
{
    char templine[1000];
    
    msglines = 0;
    while (fgets (templine, 1000, stdin))
    {
	if (msglines >= MAXLINES)
	    fprintf (thelogfile, "WARNING: message truncated @ %s\n",
		     listinfo.datestr);
	else
	    rawmsg[msglines] = strdup(templine);

	msglines ++;
    }
}

/*****************************************************************************
 * fold_if_needed
 *
 * RFC 822 allows for mail header lines to be extended to the next line if
 * the next line starts with a linear-whitespace (non-CR/LF whitespace).
 * This function concatenates such lines together.
 */
char
    *fold_if_needed (l)
int *l;
{
    char *newline, *carp;

    newline = malloc (200);
    carp = rm_tail_cr (rawmsg[*l]);
    strcpy (newline, carp);
    free (carp);

    if (islwsp (rawmsg[(*l)+1][0]))
    {
	*l = *l + 1;
	strcat (newline, &(rawmsg[*l][1]));
    }

    return (newline);
}

/*****************************************************************************
 * process_raw_msg
 *
 * Process the raw incoming message: Read the header and extract needed
 * information, discard useless lines, and save lines that are unknown for 
 * resending if the message is to go to a reflector.
 */
void
    process_raw_msg()
{
    int hdrflag = True;
    char *carp;
    int i, j;
    char *tlfrom, *line, *tcp;

    for (i=0; i<msglines; i++)
    {
	if (!(hdrflag))
	{
	    outmsg.msgbody[outmsg.mblcnt] = strdup (rawmsg[i]);
	    outmsg.mblcnt ++;
	    continue;
	}

	if (strlen (rawmsg[i]) == 1)
	{
	    hdrflag = False;
	    continue;
	}

	if ((carp = strstr (rawmsg[i], "Reply-To:")) &&
	    (carp == rawmsg[i]))
	{
	    line = fold_if_needed (&i);

	    outmsg.from = get_return_address (line);
	    tlfrom = get_long_return_address (line);

	    if (outmsg.longfrom)
	    {
		if (strlen (outmsg.longfrom) <= strlen (tlfrom))
		{
		    free (outmsg.longfrom);
		    outmsg.longfrom = tlfrom;
		}
	    }
	    else
	    {
		outmsg.longfrom = tlfrom;
	    }

	    free (line);
	    continue;
	}

	if ((carp = strstr (rawmsg[i], "From:")) &&
	    (carp == rawmsg[i]))
	{
	    line = fold_if_needed (&i);

	    tlfrom = get_long_return_address (line);

	    if (outmsg.from == NULL)
		outmsg.from = get_return_address (line);

	    if (outmsg.longfrom)
	    {
		if (strlen (outmsg.longfrom) <= strlen (tlfrom))
		{
		    free (outmsg.longfrom);
		    outmsg.longfrom = tlfrom;
		}
	    }
	    else
	    {
		outmsg.longfrom = tlfrom;
	    }
	    
	    free (line);
	    continue;
	}

	if ((carp = strstr (rawmsg[i], "Subject:")) &&
	    (carp == rawmsg[i]))
	{
	    line = fold_if_needed (&i);
	    tcp = rest_of_line (line);
	    
	    outmsg.subj = cleanup_string (tcp);

	    free (line);
	    free (tcp);
	    continue;
	}

	if ((carp = strstr (rawmsg[i], "Newsgroups:")) &&
	    (carp == rawmsg[i]))
	{
	    line = fold_if_needed (&i);
	    tcp = rest_of_line (line);

	    outmsg.newsgroup = cleanup_string (tcp);

	    free (line);
	    free (tcp);
	    continue;
	}

	if ((carp = strstr (rawmsg[i], "Apparently")) ||
	    (carp = strstr (rawmsg[i], "Cc")) ||
	    (carp = strstr (rawmsg[i], "Content")) ||
	    (carp = strstr (rawmsg[i], "Date")) ||
	    (carp = strstr (rawmsg[i], "Errors")) ||
	    (carp = strstr (rawmsg[i], "From ")) ||
	    (carp = strstr (rawmsg[i], "In-Reply-To")) ||
	    (carp = strstr (rawmsg[i], "Message-Id")) ||
	    (carp = strstr (rawmsg[i], "Mime")) ||
	    (carp = strstr (rawmsg[i], "Nntp")) ||
	    (carp = strstr (rawmsg[i], "Path")) ||
	    (carp = strstr (rawmsg[i], "Precedence")) ||
	    (carp = strstr (rawmsg[i], "Received")) ||
	    (carp = strstr (rawmsg[i], "References")) ||
	    (carp = strstr (rawmsg[i], "Reply-To")) ||
	    (carp = strstr (rawmsg[i], "Return-Path")) ||
	    (carp = strstr (rawmsg[i], "Sender")) ||
	    (carp = strstr (rawmsg[i], "Status")) ||
	    (carp = strstr (rawmsg[i], "To:")) ||
	    (carp = strstr (rawmsg[i], "X-")))
	{
	    if (carp == rawmsg[i])
	    {
		line = fold_if_needed (&i);
		free (line);
		continue;
	    }
	}

	if (outmsg.ahlcnt >= MAXALTHDR)
	{
	    fprintf (thelogfile, 
		     "Warning: Alternate header buffer overflowed @ %s\n",
		     listinfo.datestr);
	}
	else
	{
	    outmsg.althdr[outmsg.ahlcnt++] = rm_tail_cr (rawmsg[i]);
	    while (islwsp (rawmsg[i+1][0]))
	    {
		i++;

		if (outmsg.ahlcnt >= MAXALTHDR)
		    fprintf (thelogfile, 
			     "WARNING: Alt. header buffer overflowed @ %s\n",
			     listinfo.datestr);
		else
		    outmsg.althdr[outmsg.ahlcnt++] = rm_tail_cr (rawmsg[i]);
	    }
	}
    }

    outmsg.replyto = strdup (listinfo.replyto);
    outmsg.errorsto = strdup (listinfo.errorsto);
}

