
#define MAIN
#include "mailserv.h"

/*****************************************************************************
 * usage
 *
 * How to use this program.
 */
void
    usage()
{
    fprintf (stderr, "Usage: mailserv listname command < mailfile\n\n");
    fprintf (stderr, "Commands:\n");
    fprintf (stderr, "\trequest\t\tmail from request address\n");
    fprintf (stderr, "\tmessage\t\tmail from message address\n");
    fprintf (stderr, "\tnightly\t\tdo nightly maintenance\n");
    fprintf (stderr, "\trare\t\tdo infrequent maintenance\n");
    Exit (1);
}

/*****************************************************************************
 * msgdate
 *
 * Create a text string with the date of the message.  
 *
 * WARNING: This may not be creating the string required by the RFC for
 * digestifying messages correctly.
 */
char *msgdate()
{
    long t;

    t = time(0);
    return (cleanup_string (ctime (&t)));
}

/*****************************************************************************
 * init_globals
 *
 * Set a few global variables that may be checked before being set.
 */
void
    init_globals ()
{
    listinfo.datestr = msgdate();

    listinfo.delay = 5;

    listinfo.replyto = NULL;
    listinfo.errorsto = NULL;

    listinfo.keep = False;
    listinfo.announce = False;
    listinfo.files = False;
    listinfo.digest = False;
    listinfo.backup = True;
    listinfo.faq = False;
    listinfo.ftp = False;

    outmsg.from = NULL;
    outmsg.longfrom = NULL;
}

/*****************************************************************************
 * main
 */
int
    main (argc, argv)
int argc;
char **argv;
{
    init_globals ();
    get_list_info (argc, argv);
    verify_info ();
    get_user_list ();

    if (strstr (listinfo.sub, "request"))
    {
	get_raw_msg ();
	process_raw_msg ();
	handle_request ();
    }
    else if (strstr (listinfo.sub, "message"))
    {
	get_raw_msg ();
	process_raw_msg ();
	distribute_message ();
    }
    else if (strstr (listinfo.sub, "nightly"))
    {
	if (listinfo.backup)
	    backup_user_list ();

	if (listinfo.digest)
	{
	    make_digest ();

	    if (listinfo.ftp)
		ftp_digest ();
	}

	dump_log ();
    }
    else if (strstr (listinfo.sub, "rare"))
    {
	if (listinfo.faq)
	    post_faq ();
    }
    else
    {
	fprintf (stderr, "Illegal sublist type: %s\n", listinfo.sub);
	Exit(1);
    }

    Exit (0);
}
