
#ifndef MAILSERV
#define MAILSERV

/*****************************************************************************
 * Header files
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <ctype.h>

/*****************************************************************************
 * Constants
 */
#define True       1
#define False      0
#define MAXLINES   10000
#define MAXALTHDR  10
#define BADADDR    "Bad_Address"

/* File names & locations */
#define ListHome   "/usr/people/MailServ"
#define InfoFile   "Info"

/* Shell Commands */
#define SendMail   "/usr/lib/sendmail"
#define Sort       "/bin/sort"
#define Copy       "/bin/cp"
#define Rm         "/bin/rm -f"
#define Inews      "/usr/local/bin/inews"
#define Ftp        "/usr/ucb/ftp"

/* Fake address for Quiet mailings */
#define FakeAddr   "notreal@fakesite.org"

/*****************************************************************************
 * Type definitions
 */
typedef struct
{
    char *name, *sub;
    char *replyto, *errorsto, *from;
    char *archivefile, *userfile, *logfile, *indexfile, *backupdir;
    char *subscribefile, *unsubscribefile, *nounsubfile, *helpfile;
    char *digestinput, *digestoutput, *digestcounterfile;
    char *digestname, *digesthead, *digesttail;
    char *datestr, *volline;
    char *faqfile;
    char *ftpdir, *ftpsite, *netrcfile;

    int volume, issue;
    int delay;

    unsigned announce : 1;
    unsigned backup : 1;
    unsigned digest : 1;
    unsigned faq : 1;
    unsigned files : 1;
    unsigned ftp : 1;
    unsigned keep : 1;
}
ListInfo;

typedef struct
{
    char *replyto;
    char *errorsto;
    char *from, *longfrom;
    char *subj;
    char *newsgroup;

    char *althdr[MAXALTHDR];
    int ahlcnt;

    char *msgbody[MAXLINES];
    int mblcnt;
}
Message;

typedef struct
{
    char *request;
    char *filename;
}
ReqFile;
	

/*****************************************************************************
 * Global variables
 */
#ifdef MAIN
#define SCOPE
#else
#define SCOPE extern
#endif

SCOPE char *rawmsg[MAXLINES];
SCOPE int msglines;

SCOPE ListInfo listinfo;
SCOPE Message outmsg;

SCOPE char *readers[MAXLINES];
SCOPE int readercount;

SCOPE ReqFile reqtable[MAXLINES];
SCOPE int reqcount;

SCOPE FILE *thelogfile;

#endif /* #ifndef MAILSERV... */

/*****************************************************************************
 * Functions predefs, for utils
 */

FILE *Fopen (char *, char *);

char *cleanup_string (char *);
char *first_word (char *);
char *get_long_return_address (char *);
char *get_return_address (char *);
char *rest_of_line (char *);
char *rm_head_space (char *);
char *rm_tail_cr (char *);

int islwsp (char);

void Exit (int);
void backup_user_list ();
void get_user_list ();
void post_faq ();
void send_file (char *, char *, char *);
