
#include "mailserv.h"

/*****************************************************************************
 * dump_clean_message
 *
 * Output a processed message.
 */
void
    dump_clean_message(fileid)
FILE *fileid;
{
    int i;

    if (fileid)
    {
	fprintf (fileid, "Errors-To: %s\n", outmsg.errorsto);
	fprintf (fileid, "Reply-To: %s\n", outmsg.replyto);
	fflush (fileid);
    }
    else
    {
	printf ("Errors-To: %s\n", outmsg.errorsto);
	printf ("Reply-To: %s\n", outmsg.replyto);
	fflush (stdout);
    }

    if (outmsg.longfrom)
    {
	if (fileid)
	{
	    fprintf (fileid, "From: %s\n", outmsg.longfrom);
	    fflush (fileid);
	}
	else
	{
	    printf ("From: %s\n", outmsg.longfrom);
	    fflush (stdout);
	}
    }

    if (outmsg.subj)
    {
	if (fileid)
	{
	    fprintf (fileid, "Subject: %s\n", outmsg.subj);
	    fflush (fileid);
	}
	else
	{
	    printf ("Subject: %s\n", outmsg.subj);
	    fflush (stdout);
	}
    }

    for (i=0; i<outmsg.ahlcnt; i++)
    {
	if (fileid)
	{
	    fprintf (fileid, "%s\n", outmsg.althdr[i]);
	    fflush (fileid);
	}
	else
	{
	    printf ("%s\n", outmsg.althdr[i]);
	    fflush (stdout);
	}
    }

    if (fileid)
    {
	fprintf (fileid, "List-Name: %s\n", listinfo.name);
	fprintf (fileid, "Precedence: Bulk\n");
	fprintf (fileid, "\n");
	fflush (fileid);
    }
    else
    {
	printf ("List-name: %s\n", listinfo.name);
	printf ("Precedence: Bulk\n");
	printf ("\n");
	fflush (stdout);
    }

    if (outmsg.newsgroup)
    {
	if (fileid)
	{
	    fprintf (fileid, "Newsgroups: %s\n", outmsg.newsgroup);
	    fflush (fileid);
	}
	else
	{
	    printf ("Newsgroups: %s\n", outmsg.newsgroup);
	    fflush (stdout);
	}
    }

    for (i=0; i<outmsg.mblcnt; i++)
    {
	if (fileid)
	{
	    fprintf (fileid, "%s", outmsg.msgbody[i]);
	    fflush (fileid);
	}
	else
	{
	    printf ("%s", outmsg.msgbody[i]);
	    fflush (stdout);
	}
    }
}

/*****************************************************************************
 * dump_quiet_clean_message
 *
 * Output a processed message with the changes required to mail it "quietly"
 * (ie: the "To:" line doesn't contain the receiver's address).
 */
void
    dump_quiet_clean_message(fileid, qto)
FILE *fileid;
char *qto;
{
    int i;

    if (fileid)
    {
	fprintf (fileid, "Errors-To: %s\n", outmsg.errorsto);
	fprintf (fileid, "Reply-To: %s\n", outmsg.replyto);
	fprintf (fileid, "Bcc: %s\n", qto);
	fflush (fileid);
    }
    else
    {
	printf ("Errors-To: %s\n", outmsg.errorsto);
	printf ("Reply-To: %s\n", outmsg.replyto);
	printf ("Bcc: %s\n", qto);
	fflush (stdout);
    }

    if (outmsg.longfrom)
    {
	if (fileid)
	{
	    fprintf (fileid, "From: %s\n", outmsg.longfrom);
	    fflush (fileid);
	}
	else
	{
	    printf ("From: %s\n", outmsg.longfrom);
	    fflush (stdout);
	}
    }

    if (outmsg.subj)
    {
	if (fileid)
	{
	    fprintf (fileid, "Subject: %s\n", outmsg.subj);
	    fflush (fileid);
	}
	else
	{
	    printf ("Subject: %s\n", outmsg.subj);
	    fflush (stdout);
	}
    }

    for (i=0; i<outmsg.ahlcnt; i++)
    {
	if (fileid)
	{
	    fprintf (fileid, "%s\n", outmsg.althdr[i]);
	    fflush (fileid);
	}
	else
	{
	    printf ("%s\n", outmsg.althdr[i]);
	    fflush (stdout);
	}
    }

    if (fileid)
    {
	fprintf (fileid, "List-Name: %s\n", listinfo.name);
	fprintf (fileid, "Precedence: Bulk\n");
	fprintf (fileid, "\n");
	fflush (fileid);
    }
    else
    {
	printf ("List-name: %s\n", listinfo.name);
	printf ("Precedence: Bulk\n");
	printf ("\n");
	fflush (stdout);
    }

    if (outmsg.newsgroup)
    {
	if (fileid)
	{
	    fprintf (fileid, "Newsgroups: %s\n", outmsg.newsgroup);
	    fflush (fileid);
	}
	else
	{
	    printf ("Newsgroups: %s\n", outmsg.newsgroup);
	    fflush (stdout);
	}
    }

    for (i=0; i<outmsg.mblcnt; i++)
    {
	if (fileid)
	{
	    fprintf (fileid, "%s", outmsg.msgbody[i]);
	    fflush (fileid);
	}
	else
	{
	    printf ("%s", outmsg.msgbody[i]);
	    fflush (stdout);
	}
    }
}

/*****************************************************************************
 * dump_raw_msg
 *
 * Output an unprocessed message.
 */
void
    dump_raw_msg(fileid)
FILE *fileid;
{
    int i;

    for (i=0; i<msglines; i++)
    {
	if (fileid)
	{
	    fputs (rawmsg[i], fileid);
	    fflush (fileid);
	}
	else
	{
	    fputs (rawmsg[i], stdin);
	    fflush (stdout);
	}
    }
}   

/*****************************************************************************
 * send_to_digest
 *
 * Store the (unprocessed) message for later processing by the digestifier.
 */
void
    send_to_digest()
{
    FILE *digestfile;

    digestfile = Fopen (listinfo.digestinput, "a");
    dump_raw_msg (digestfile);
    fclose (digestfile);
}

/*****************************************************************************
 * send_to_users
 *
 * Reflect the (processed) message to users on the list.
 */
void
    send_to_users()
{
    FILE *outfile;
    char cmdstr[120], *carp;
    int i;

    for (i=0; i<readercount; i++)
    {
	if (readers[i][0] == '*')
	{
	    sprintf (cmdstr, "%s %s", SendMail, FakeAddr);
	    outfile = popen (cmdstr, "w");
	    carp = rest_of_line (readers[i]);
	    dump_quiet_clean_message (outfile, carp);
	}
	else
	{
	    sprintf (cmdstr, "%s %s", SendMail, readers[i]);
	    outfile = popen (cmdstr, "w");
	    dump_clean_message (outfile);
	}

	pclose (outfile);
	sleep (listinfo.delay);
    }
}

/*****************************************************************************
 * archive_message
 *
 * Save a copy of the (processed) message to the archive file.
 *
 * WARNING: The date of the message isn't be written to the archive file
 * right now.  This needs to be fixed.
 */
void
    archive_message()
{
    FILE *outfile;

    outfile = Fopen (listinfo.archivefile, "a");
    dump_clean_message (outfile);
    fclose (outfile);
}

/*****************************************************************************
 * distribute_message
 *
 * Figure out what to do with a message; save it for digestifying later, 
 * process it and mail it out to the readers, archive it, etc., etc.
 */
void
    distribute_message()
{
    fprintf (thelogfile, "Message: %s @ %s\n",
	     outmsg.from, listinfo.datestr);
    fprintf (thelogfile, "     Subj: %s\n",
	     outmsg.subj);

    if (outmsg.newsgroup)
	fprintf (thelogfile, "     News: %s\n", outmsg.newsgroup);

    if (listinfo.digest)
    {
	send_to_digest();
	return;
    }

    send_to_users();

    if (listinfo.keep)
	archive_message();

}
