
#include "mailserv.h"

/*****************************************************************************
 * std_req_hdr
 *
 * Slap a header on the reply to a request.  Set the precedence for low
 * priority mailing, set return addresses for bounces.  If you don't set
 * the return addresses, all the bounces go to the sys admins, and they're
 * probably busy enough.
 *
 * NOTE: Perhaps the precedence level should be an entry in the info file,
 * instead of hard coding it.
 */
void
    std_req_hdr (fileid)
FILE *fileid;
{
    if (fileid)
    {
        fprintf (fileid, "Errors-To: %s\n", outmsg.errorsto);
        fprintf (fileid, "Reply-To: %s\n", listinfo.from);
	fprintf (fileid, "From: %s\n", listinfo.from);
        fprintf (fileid, "List-Name: %s\n", listinfo.name);
        fprintf (fileid, "Precedence: Bulk\n\n");
	fflush (fileid);
    }
    else
    {
        printf ("Errors-To: %s\n", outmsg.errorsto);
        printf ("Reply-To: %s\n", listinfo.from);
	printf ("From: %s\n", listinfo.from);
	printf ("List-name: %s\n", listinfo.name);
        printf ("Precedence: Bulk\n\n");
	fflush (stdout);
    }
}

/*****************************************************************************
 * quiet_req_hdr
 *
 * Make a header for "quiet" mailings.  This is rather useless right now,
 * as users can't make themselves quiet; the list maintainer has to do that.
 *
 * NOTE: A new command needs to be added to allow users to make themselves
 * marked as quiet on the list.  Also, there needs to be an extra command
 * on the request (subject) line so that requests will also be sent back
 * quiet.
 */
void
    quiet_req_hdr (fileid, qto)
FILE *fileid;
char *qto;
{
    if (fileid)
    {
        fprintf (fileid, "Errors-To: %s\n", outmsg.errorsto);
        fprintf (fileid, "Reply-To: %s\n", listinfo.from);
	fprintf (fileid, "From: %s\n", listinfo.from);
	fprintf (fileid, "Bcc: %s\n", qto);
        fprintf (fileid, "Precedence: Bulk\n\n");
	fflush (fileid);
    }
    else
    {
        printf ("Errors-To: %s\n", outmsg.errorsto);
        printf ("Reply-To: %s\n", listinfo.from);
	printf ("From: %s\n", listinfo.from);
	printf ("Bcc: %s\n", qto);
        printf ("Precedence: Bulk\n\n");
	fflush (stdout);
    }
}

/*****************************************************************************
 * handle_bad_address
 *
 * This gets called if the program can't figure out a reasonable return
 * address.
 */
void
    handle_bad_address()
{
    FILE *cmdstream;
    char cmdstr[100];

    sprintf (cmdstr, "%s %s-owner", SendMail, listinfo.name);
    cmdstream = popen (cmdstr, "w");

    sprintf (cmdstr, "Subject: BAD ADDRESS in %s\n", listinfo.name);
    fputs (cmdstream, cmdstr);
    std_req_hdr (cmdstream);

    dump_raw_msg (cmdstream);
    pclose (cmdstream);
}

/*****************************************************************************
 * make_request_table
 * 
 * Read the index file, and make a table to respond to requests for files.
 */
void
    make_request_table ()
{
    char templine[100], *carp;
    char request[40], thefile[40];
    FILE *reqfile;
    
    reqfile = Fopen (listinfo.indexfile, "r");

    while (fgets (templine, 100, reqfile) != NULL)
    {
	carp = cleanup_string (templine);

        if (carp[0] != '#')
	{
	    int i;

	    reqtable[reqcount].request = (char *)
		cleanup_string (first_word (carp));
	    reqtable[reqcount].filename = (char *)
		cleanup_string (rest_of_line (carp));
	    for (i=0; i<strlen(reqtable[reqcount].request); i++)
		if (isupper (reqtable[reqcount].request[i]))
		    request[i] = tolower(reqtable[reqcount].request[i]);
	    reqcount++;
	}
    }

    fclose (reqfile);
}

/*****************************************************************************
 * announce
 *
 * If the announce flag is set, tell everyone on the list when someone
 * subscribes or unsubscribes.
 */
void
    announce (annstring)
char *annstring;
{
    FILE *cmdstream;
    char cmdstr[100];

    sprintf (cmdstr, "%s %s", SendMail, listinfo.name);
    cmdstream = popen (cmdstr, "w");

    fprintf (cmdstream, "Subject: %s", annstring);
    std_req_hdr (cmdstream);
    fprintf (cmdstream, "%s\n", annstring);
	
    pclose (cmdstream);
}

/*****************************************************************************
 * handle_subscribe
 *
 * Run when a subscribe request comes in.
 */
void
    handle_subscribe ()
{
    char filepath[100];
    FILE *fileptr;

    fileptr = Fopen (listinfo.userfile, "a");
    fprintf (fileptr, "%s\n", outmsg.from);
    fclose (fileptr);

    send_file (outmsg.from, listinfo.subscribefile, "Subscribed");

    fprintf (thelogfile, "Subscribed: %s @ %s\n",
	     outmsg.from, listinfo.datestr);
    fprintf (thelogfile, "     Subj: %s\n", outmsg.subj);

    if (listinfo.announce)
    {
	sprintf (filepath, "Adding %s to the %s list.\n",
		 outmsg.from, listinfo.name);
	announce (filepath);
    }

    if (listinfo.digest)
	send_file (outmsg.from, listinfo.digestoutput, "Previous Digest");
}

/*****************************************************************************
 * handle_unsubscribe
 *
 * Run when an unsubscribe request comes in.
 */
void
    handle_unsubscribe ()
{
    int i, ok = False, thisuser=0;
    char filepath[100], *carp;
    FILE *fileptr;

    for (i=0; i<readercount; i++)
    {
	if ((carp = strstr (readers[i], outmsg.from)) &&
	    (carp == readers[i]))
	{
	    thisuser = i;
	    ok = True;
	    break;
	}
	else if ((carp = strstr (readers[i], outmsg.from)) &&
		 (carp == (readers[i]) + 2) &&
		 (readers[i][0] == '*'))
	{
	    thisuser = i;
	    ok = True;
	    break;
	}

    }

    if (ok)
    {
	fileptr = Fopen (listinfo.userfile, "w");

	for (i=0; i<readercount; i++)
	{
	    if (i != thisuser)
		fprintf (fileptr, "%s\n", readers[i]);
	    fflush (fileptr);
	}
	fclose (fileptr);

	send_file (outmsg.from, listinfo.unsubscribefile, "Unsubscribed");

	fprintf (thelogfile, "Unsubscribed: %s @ %s\n",
		 outmsg.from, listinfo.datestr);
	fprintf (thelogfile, "     Subj: %s\n", outmsg.subj);

	if (listinfo.announce)
	{
	    sprintf (filepath, "Removing %s from the %s list.\n",
		     outmsg.from, listinfo.name);
	    announce (filepath);
	}
    }
    else
    {
	send_file (outmsg.from, listinfo.nounsubfile, "Unsubscribe FAILED");

	fprintf (thelogfile, "Didn't unsub %s @ %s\n", 
		 outmsg.from, listinfo.datestr);
	fprintf (thelogfile, "     Subj: %s\n", outmsg.subj);
    }
}		    

/*****************************************************************************
 * handle_show_list
 *
 * Run when a show-list request comes in.
 */
void
    handle_show_list ()
{
    char filepath[100];

    send_file (outmsg.from, listinfo.userfile, "User List");

    fprintf (thelogfile, "Sent userlist to: %s @ %s\n", 
	     outmsg.from, listinfo.datestr);
    fprintf (thelogfile, "     Subj: %s\n", outmsg.subj);
}

/*****************************************************************************
 * handle_file_request
 *
 * Run when a file is requested as listed in the index file.
 */
void
    handle_file_request (filenum)
int filenum;
{
    send_file (outmsg.from, reqtable[filenum].filename,
	       reqtable[filenum].request);

    fprintf (thelogfile, "Sent %s to: %s @ %s\n", 
	     reqtable[filenum].filename, outmsg.from, listinfo.datestr);
    fprintf (thelogfile, "     Subj: %s\n", outmsg.subj);
}

/*****************************************************************************
 * handle_help
 *
 * Run when the request command is not understood.
 */
void
    handle_help ()
{
    send_file (outmsg.from, listinfo.helpfile, "Help");

    fprintf (thelogfile, "Helping: %s @ %s\n", 
	     outmsg.from, listinfo.datestr);
    fprintf (thelogfile, "     Subj: %s\n", outmsg.subj);
}

/*****************************************************************************
 * handle_request
 *
 * Try to figure out what the user is requesting.
 */
void
    handle_request()
{
    char *request, *carp;
    int i;

    if (strstr (outmsg.from, BADADDR))
	handle_bad_address();

    if (listinfo.files)
	make_request_table();

    if (outmsg.subj == NULL)
    {
	handle_help();
	return;
    }

    request = strdup (outmsg.subj);
    for (i=0; i<strlen(request); i++)
	if (isupper (request[i]))
	    request[i] = tolower(request[i]);

    if ((carp = strstr (request, "unsubscribe")) &&
	(carp == request))
    {
	handle_unsubscribe();
	return;
    }

    if ((carp = strstr (request, "subscribe")) &&
	(carp == request))
    {
	handle_subscribe();
	return;
    }

    if ((carp = strstr (request, "show-list")) &&
	(carp == request))
    {
	if (listinfo.announce)
	    handle_show_list ();
	else
	    handle_help ();

	return;
    }

    if (listinfo.files)
    {
	for (i=0; i<reqcount; i++)
	{
	    if ((carp = strstr (request, reqtable[i].request)) &&
		(carp == request))
	    {
		handle_file_request (i);
		return;
	    }
	}
    }

    handle_help ();

}
