
#include "mailserv.h"

/*****************************************************************************
 * Exit
 *
 * Secondary exit function.  Flush important buffers, then exit.
 */
void
    Exit (state)
int state;
{
    fflush (stderr);

    if (thelogfile != NULL)
	fclose (thelogfile);

    exit (state);
}

/*****************************************************************************
 * Fopen
 *
 * Secondary fopen function.  Make sure the file is actually being opened
 * so the program doesn't crash later.
 */
FILE
    *Fopen (filename, type)
char *filename, *type;
{
    FILE *tfp;

    if ((tfp = fopen (filename, type)) == NULL)
    {
	if (thelogfile != NULL)
	{
	    fprintf (thelogfile, "FATAL ERROR: Couldn't open file %s for ",
		     filename);
	    if (type[0] == 'a')
		fprintf (thelogfile, "appending.\n");
	    else if (type[0] == 'r')
		fprintf (thelogfile, "reading.\n");
	    else if (type[0] == 'w')
		fprintf (thelogfile, "writing.\n");
	    else
		fprintf (thelogfile, "unknown operation %s\n", type);
	}

	fprintf (stderr, "FATAL ERROR: Couldn't open file %s for ", filename);
	if (type[0] == 'a')
	    fprintf (stderr, "appending.\n");
	else if (type[0] == 'r')
	    fprintf (stderr, "reading.\n");
	else if (type[0] == 'w')
	    fprintf (stderr, "writing.\n");
	else
	    fprintf (stderr, "unknown operation %s\n", type);

	Exit(1);
    }

    return (tfp);
}

/*****************************************************************************
 * islwsp
 *
 * Is the character linear-whitespace (non CR/LF whitespace)?
 */
int
    islwsp(c)
char c;
{
    char cr[2];

    if (isspace (c))
    {
	sprintf (cr, "\n");
	if (c != cr[0])
	    return (True);
    }

    return (False);
}

/*****************************************************************************
 * first_word
 *
 * Return the first word in a string.
 */
char
    *first_word (line)
char *line;
{
    char *tcp, tc;
    int i=0;

    while (i < strlen(line))
    {
	if (isspace (line[i]))
	{
	    tc = line[i];
	    line[i] = '\0';
	    tcp = strdup (line);
	    line[i] = tc;
	    return (tcp);
	}
	i++;
    }

    return (strdup (line));
}

/*****************************************************************************
 * rest_of_line
 *
 * Return everything but the first word in a string.
 */
char
    *rest_of_line (line)
char *line;
{
    int i=0;

    while (i < strlen(line))
    {
	if (isspace (line[i]))
	{
	    return (strdup ( &(line[i+1]) ));
	    break;
	}
	i++;
    }

    return (NULL);
}

/*****************************************************************************
 * rm_tail_cr
 *
 * Remove any trailing CR/LF's on a line, if they exist.
 */
char
    *rm_tail_cr (line)
char *line;
{
    char *carp, *tcp;
    char cr[2];

    sprintf (cr, "\n");

    tcp = strdup (line);
    if (carp = strstr (tcp, cr))
	*carp = '\0';

    return (tcp);
}

/*****************************************************************************
 * rm_head_space
 *
 * Make sure a line starts with a visible character.
 */
char
    *rm_head_space (line)
char *line;
{
    int flag = True;
    char *tcp, *carp1, *carp2;

    carp1 = tcp = strdup (line);

    while (flag)
    {
	if (isspace (*carp1))
	    carp1++;
	else
	    flag = False;
    }

    carp2 = strdup (carp1);
    free (tcp);

    return (carp2);
}

/*****************************************************************************
 * cleanup_string
 *
 * Make a string "clean"; no leading whitespace, no trailing CR/LF's, etc.
 */
char
    *cleanup_string (line)
char *line;
{
    char *tc1, *tc2;

    tc1 = rm_tail_cr (line);
    tc2 = rm_head_space (tc1);
    free (tc1);
    return (tc2);
}

/*****************************************************************************
 * get_return_address
 *
 * Given a Reply-To: or From: line, figure out a return address.
 */
char 
    *get_return_address (inln)
char *inln;
{
    int i, lng, ci;
    char *line;
    char *carp, *tcp, *res;

    line = rest_of_line (inln);    

    if ((carp = strstr (line, "<")))
    {
	carp++;
	tcp = strstr (line, ">");
	*tcp = '\0';
	return (strdup (carp));
    }

    lng = strlen(line);
    carp = malloc (lng);
    ci = 0;

    for (i=0; i<lng; i++)
    {
	if (line[i] == '(')
	{
	    while (line[i] != ')')
		i++;
	    continue;
	}

	if (line[i] == '"')
	{
	    i++;
	    while (line[i] != '"')
		i++;
	    continue;
	}

	if (line[i] == '[')
	{
	    while (line[i] != ']')
		i++;
	    continue;
	}

	carp[ci++] = line[i];
    }

    carp[ci] = '\0';
    tcp = cleanup_string (carp);
    free (carp);
    return (tcp);
}

/*****************************************************************************
 * get_long_return_address
 *
 * Instead of just the return address, this gives everything on a From: or
 * Reply-To: line.
 */
char
    *get_long_return_address (line)
char *line;
{
    char *tc1, *tc2;

    tc1 = rest_of_line (line);
    tc2 = cleanup_string (tc1);
    free (tc1);
    return (tc2);
}

/*****************************************************************************
 * send_file
 *
 * Send a file to a user.
 */
void
    send_file (name, filepath, subj)
char *name, *filepath, *subj;
{
    FILE *cmdstream, *datafile;
    char cmdstr[100];

    if (filepath[0] == '/')
	sprintf (cmdstr, "%s", filepath);
    else
	sprintf (cmdstr, "%s/%s/%s", ListHome, listinfo.name, filepath);

    datafile = Fopen (cmdstr, "r");

    sprintf (cmdstr, "%s %s", SendMail, name);
    cmdstream = popen (cmdstr, "w");

    fprintf (cmdstream, "Subject: %s\n", subj);
    std_req_hdr (cmdstream);

    while (fgets (cmdstr, 100, datafile) != NULL)
    {
	fputs (cmdstr, cmdstream);
	fflush (cmdstream);
    }

    pclose (cmdstream);
    fclose (datafile);
}

/*****************************************************************************
 * get_user_list
 *
 * Read in the user list.
 */
void
    get_user_list()
{
    char templine[100], *carp;
    FILE *userfile;

    readercount = 0;
    userfile = Fopen (listinfo.userfile, "r");
    while (fgets (templine, 100, userfile) != NULL)
    {
	if (templine[0] != '#')
	{
	    if (readercount > MAXLINES)
	    {
		fprintf (stderr, "Warning: User list overflow.\n");
		fprintf (thelogfile, "Warning: User list overflow @ %s\n",
			 listinfo.datestr);
		break;
	    }

	    readers[readercount++] = cleanup_string (templine);
	}
    }
}

/*****************************************************************************
 * backup_user_list
 *
 * Backup the user list.  Append the date to the filename.  Store the last
 * 7 days of lists.
 */
void
    backup_user_list()
{
    char tl[200];
    int d;

    d = atoi (&(listinfo.datestr[8]));

    if (listinfo.backup)
    {
	sprintf (tl, "%s -u -o %s %s", Sort,
		 listinfo.userfile, listinfo.userfile);
	system (tl);

	sprintf (tl, "%s %s %s/%d", Copy, listinfo.userfile,
		 listinfo.backupdir, d);
	system (tl);

	d = (d + 31 - 7) % 31;
	sprintf (tl, "%s -f %s/%d", Rm, listinfo.backupdir, d);
	system (tl);
    }
}

/*****************************************************************************
 * dump_log
 *
 * Dump the logfile, and erase it.
 */
void
    dump_log ()
{
    char templine[200];

    fclose (thelogfile);
    thelogfile = Fopen (listinfo.logfile, "r");

    printf ("==================================\n");
    printf ("List Log: %s    (%d users)\n\n", listinfo.name, readercount);

    while (fgets (templine, 200, thelogfile))
	fputs (templine, stdout);

    fclose (thelogfile);
    printf ("\n");
    thelogfile = Fopen (listinfo.logfile, "w");
}

/*****************************************************************************
 * post_faq
 *
 * Post a FAQ to USENET.
 */
void
    post_faq ()
{
    char cmdstr[100];

    sprintf (cmdstr, "%s %s", Inews, listinfo.faqfile);
    system (cmdstr);
}

/*****************************************************************************
 * ftp_digest
 *
 * Put the digest up on an ftp site.
 */
void
    ftp_digest ()
{
    char cmd[200];
    FILE *cmdstream;

    sprintf (cmd, "%s %s %s/.netrc", Copy, listinfo.netrcfile, ListHome);
    system (cmd);

    sprintf (cmd, "%s %s", Ftp, listinfo.ftpsite);
    cmdstream = popen (cmd, "w");
    sleep (15);

    fprintf (cmdstream, "bin\n");
    sleep (10);

    fprintf (cmdstream, "prompt\n");
    sleep (10);

    fprintf (cmdstream, "put %s  %s/daily\n", 
	     listinfo.digestoutput, listinfo.ftpdir);
    sleep (10);

    fprintf (cmdstream, "put %s  %s/vol-%d/%d\n",
	     listinfo.digestoutput, listinfo.ftpdir, 
	     listinfo.volume, listinfo.issue);
    sleep (10);
    
    fprintf (cmdstream, "quit\n");
    sleep (100);

    sprintf (cmd, "%s %s/.netrc", Rm, ListHome);
    system (cmd);

    pclose (cmdstream);
}
